/*****************************************************************************/
/*                      WiNRADiO G69DDC EXCELSIOR ULTRA                      */
/*                                SDK DEMO                                   */
/*                      FFT and FFT overlapping buffer                       */
/*                        (c) Copyright RADIXON 2022                         */
/*****************************************************************************/

#ifndef __U_FFT_H__
#define __U_FFT_H__

#include "u_defs.h"
#include "u_types.h"

class TFFTBuffer
{
public:
    bool AddSamples_16(const int16_t *Samples,uint32_t Count);
    bool AddSamples_32(const int32_t *Samples,uint32_t Count);
    bool AddSamples_F32(const float *Samples,uint32_t Count);

    inline void Reset(void)
    {
        m_Used=0;
    }

    inline void SetSize(uint32_t Size)
    {
        if(Size>m_MaxSize)
        {
            m_Size=m_MaxSize;
        }
        else
        {
            m_Size=Size;
        }

        m_Used=0;
    }

    inline const float* Get(void) const
    {
        if(m_Size!=m_Used)
        {
            return 0;
        }

        return m_Buffer;
    }

    TFFTBuffer(uint32_t MaxSize);
    ~TFFTBuffer(void);
private:
    float       *m_Buffer;
    uint32_t    m_MaxSize;
    uint32_t    m_Size;
    uint32_t    m_Used;
};

void FFT(float* data, unsigned long nn);
void GetNormalizedWindowCoeffs(float *Buffer,int Count,double FixedCoeff=1);

#endif //__U_FFT_H__
