/*****************************************************************************/
/*                      WiNRADiO G69DDC EXCELSIOR ULTRA                      */
/*                                SDK DEMO                                   */
/*                       Simple sweeper spectrum panel                       */
/*                        (c) Copyright RADIXON 2022                         */
/*****************************************************************************/

#ifndef __U_SWPSPECTRUM_H__
#define __U_SWPSPECTRUM_H__

#include <wx/wx.h>
#include "u_types.h"
#include "u_spectrum.h"

class TSwpSpectrum:public wxPanel
{
public:
    void AddData(const float *Data,uint32_t Count);
    void Start(int64_t StartFrequency,int64_t EndFrequency,double RBW);
    void Stop(void);
    void Clear(void);

    virtual bool Enable(bool enable);
    virtual bool Show(bool show=true);

    TSwpSpectrum(wxWindow *Parent,wxWindowID id,const wxPoint &Position=wxDefaultPosition,const wxSize &Size=wxDefaultSize);
    ~TSwpSpectrum(void);

private:
    int64_t             m_StartFrequency;
    int64_t             m_EndFrequency;
    int64_t             m_Range;
    double              m_RBW;
    uint32_t            m_DataCount;

    float               *m_Data;
    uint32_t            m_DataPosition;
    uint32_t            m_ChangedDataCount;

    wxPoint             *m_DisplayData;
    uint32_t            m_DisplayDataPosition;
    uint32_t            m_ValidDisplayDataCount;
    wxMutex             m_Mutex;

    bool                m_Sweeping;
    int                 m_MinLevel;
    bool                m_TuneWhenMouseMove;

    wxRect              m_GraphRect;

    double              m_Frequency2Pixel;
    double              m_Pixel2Frequency;
    double              m_FrequencyStep;

    double              m_Level2Pixel;
    double              m_Pixel2Level;
    int                 m_LevelStep;

    bool                m_InvalidDisplayData;

    void OnPaintEvent(wxPaintEvent &Event);
    void OnSizeEvent(wxSizeEvent &Event);
    void OnEraseBackgroundEvent(wxEraseEvent &Event);
    void OnDataChanged(wxNotifyEvent &Event);
    void OnMouseDown(wxMouseEvent &Event);
    void OnMouseUp(wxMouseEvent &Event);
    void OnMouseMove(wxMouseEvent &Event);
    void OnMouseCaptureLost(wxMouseCaptureLostEvent &Event);

    void UpdateParams(void);
    void UpdateDisplayData(uint32_t First,uint32_t Count);

    void Tune(const wxPoint &Pos);
    void Set(int64_t Start,int64_t End,double RBW);

    DECLARE_EVENT_TABLE()
};

#endif
