#include "g69ddcapi.h"
#include <stdio.h>
#include <math.h>
#include <dlfcn.h>
#include <inttypes.h>
#include <pthread.h>
#include <pulse/simple.h>

G69DDC_OPEN_DEVICE OpenDevice;
G69DDC_CLOSE_DEVICE CloseDevice;
G69DDC_SET_POWER SetPower;
G69DDC_SET_DDC SetDDC;
G69DDC_GET_DDC GetDDC;
G69DDC_GET_DDC_INFO GetDDCInfo;
G69DDC_SET_CALLBACKS SetCallbacks;
G69DDC_SET_FREQUENCY SetFrequency;
G69DDC_GET_FREQUENCY GetFrequency;
G69DDC_START_DDC StartDDC;
G69DDC_STOP_DDC StopDDC;
G69DDC_START_AUDIO StartAudio;
G69DDC_STOP_AUDIO StopAudio;
G69DDC_SET_AGC SetAGC;
G69DDC_SET_AGC_PARAMS SetAGCParams;
G69DDC_SET_DEMODULATOR_MODE SetDemodulatorMode;
G69DDC_SET_DEMODULATOR_FILTER_BANDWIDTH SetDemodulatorFilterBandwidth;
G69DDC_SET_DDC_FREQUENCY SetDDCFrequency;
G69DDC_GET_DDC_FREQUENCY GetDDCFrequency;
G69DDC_SET_DEMODULATOR_FREQUENCY SetDemodulatorFrequency;
G69DDC_GET_DEMODULATOR_FREQUENCY GetDemodulatorFrequency;
G69DDC_CREATE_DDC1 CreateDDC1;
G69DDC_CREATE_DDC2 CreateDDC2;
G69DDC_GET_FRONT_END_FREQUENCY GetFrontEndFrequency;

#define AUDIO_SAMPLE_RATE 48000

int32_t hDevice;
void *API=NULL;
pa_simple *PaSimple=NULL;


int Initialize(void)
{
 pa_sample_spec SampleFormat;
 int Error;
 
    API=dlopen("libg69ddcapi.so",RTLD_LAZY);

    if(API==NULL)
    {
        printf("Failed to load libg69ddcapi.so. %s\n",dlerror());
        return 0;
    }

    //retrieve addresses of used API functions
    OpenDevice=(G69DDC_OPEN_DEVICE)dlsym(API,"OpenDevice");
    CloseDevice=(G69DDC_CLOSE_DEVICE)dlsym(API,"CloseDevice");
    SetPower=(G69DDC_SET_POWER)dlsym(API,"SetPower");
    SetDDC=(G69DDC_SET_DDC)dlsym(API,"SetDDC");
    GetDDC=(G69DDC_GET_DDC)dlsym(API,"GetDDC");    
    GetDDCInfo=(G69DDC_GET_DDC_INFO)dlsym(API,"GetDDCInfo");
    SetCallbacks=(G69DDC_SET_CALLBACKS)dlsym(API,"SetCallbacks");
    SetFrequency=(G69DDC_SET_FREQUENCY)dlsym(API,"SetFrequency");
    GetFrequency=(G69DDC_GET_FREQUENCY)dlsym(API,"GetFrequency");
    StartDDC=(G69DDC_START_DDC)dlsym(API,"StartDDC");
    StopDDC=(G69DDC_STOP_DDC)dlsym(API,"StopDDC");    
    StartAudio=(G69DDC_START_AUDIO)dlsym(API,"StartAudio");
    StopAudio=(G69DDC_STOP_AUDIO)dlsym(API,"StopAudio");    
    SetAGC=(G69DDC_SET_AGC)dlsym(API,"SetAGC");
    SetAGCParams=(G69DDC_SET_AGC_PARAMS)dlsym(API,"SetAGCParams");
    SetDemodulatorMode=(G69DDC_SET_DEMODULATOR_MODE)dlsym(API,"SetDemodulatorMode");
    SetDemodulatorFilterBandwidth=(G69DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)dlsym(API,"SetDemodulatorFilterBandwidth");
    SetDDCFrequency=(G69DDC_SET_DDC_FREQUENCY)dlsym(API,"SetDDCFrequency");
    GetDDCFrequency=(G69DDC_GET_DDC_FREQUENCY)dlsym(API,"GetDDCFrequency");
    SetDemodulatorFrequency=(G69DDC_SET_DEMODULATOR_FREQUENCY)dlsym(API,"SetDemodulatorFrequency");    
    GetDemodulatorFrequency=(G69DDC_GET_DEMODULATOR_FREQUENCY)dlsym(API,"GetDemodulatorFrequency");    
    CreateDDC1=(G69DDC_CREATE_DDC1)dlsym(API,"CreateDDC1");
    CreateDDC2=(G69DDC_CREATE_DDC2)dlsym(API,"CreateDDC2");    
    GetFrontEndFrequency=(G69DDC_GET_FRONT_END_FREQUENCY)dlsym(API,"GetFrontEndFrequency");

    //open the first available G69DDC device
    hDevice=OpenDevice(G69DDC_OPEN_FIRST);

    if(hDevice<0)
    {
        printf("No available G69DDC device found.\n");
        return 0;
    }
            
    //initialize pulse audio
    SampleFormat.channels=2;
    SampleFormat.rate=AUDIO_SAMPLE_RATE;
    SampleFormat.format=PA_SAMPLE_FLOAT32LE;
    
    PaSimple=pa_simple_new(NULL,"G69DDC SDK EXAMPLE",PA_STREAM_PLAYBACK,NULL,"Audio0",&SampleFormat,NULL,NULL,&Error);
    
    if(PaSimple==NULL)
    {
        printf("Failed to initialize pulse audio.\n");
    }

    return 1;
}

void MyDDC2PreprocessedStreamCallback(uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float SlevelPeak,float SlevelRMS,uintptr_t UserData)
{
 double Slevel_dBm;
 
      //convert slevel RMS in Volts to dBm
      Slevel_dBm=10.0*log10(SlevelRMS*SlevelRMS*(1000.0/50.0));      
      printf("\rSlevel: RMS[V]=%9.7f V, RMS[dBm]=%6.1f dBm, Peak[V]=%9.7f V",SlevelRMS,Slevel_dBm,SlevelPeak);
      fflush(stdout); 
}

void MyAudioStreamCallback(uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count,uintptr_t UserData)
{
 int Error;
  
    if(Stage==2 && PaSimple!=NULL)
    {
        //we want to play only audio buffers of type 2 which means final audio samples (audio gain, audio filter and volume are applied)
        pa_simple_write(PaSimple,Buffer,Count*2*sizeof(float),&Error);
    }
}

int main(void)
{
 G69DDC_DDC_INFO DDCInfo;
 uint32_t DDCTypeIndex;
 G69DDC_CALLBACKS Callbacks;
 uint64_t Frequency;
 uint64_t FrontEndFrequency;
 int32_t DDC1Frequency;
 int32_t DDC2Frequency;
 int32_t DemodulatorFrequency; 
 uint32_t SamplesPerBuffer;
 uint32_t DDC1Id;
 uint32_t DDC2Id;

    if(!Initialize())
    {
        return -1;
    }

    //set power
    SetPower(hDevice,1);
    
    //create DDC1 channel
    CreateDDC1(hDevice,&DDC1Id);
    
    //create DDC2 channel
    CreateDDC2(hDevice,DDC1Id,&DDC2Id);

    //set DDC type of DDC1 to 1 (DDC Bandwidth=24000, SampleRate=32kHz, BitsPerSample=32)
    SetDDC(hDevice,DDC1Id,1);

    //retrieve current DDC type of the DDC1
    GetDDC(hDevice,DDC1Id,&DDCTypeIndex,&DDCInfo);    
    printf("DDC type of DDC1: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);

    //set DDC type of DDC2 to 1 (the same as DDC1)
    SetDDC(hDevice,DDC2Id,1);

    //retrieve current DDC type of the DDC2
    GetDDC(hDevice,DDC2Id,&DDCTypeIndex,&DDCInfo);    
    printf("DDC type of DDC2: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);

    //ADCSnapshotCallback is not required in this example
    Callbacks.ADCSnapshotCallback=NULL;

    //DDC1StreamCallback is not required in this example
    Callbacks.DDC1StreamCallback=NULL;

    //DDC1PlaybackStreamCallback is not required in this example
    Callbacks.DDC1PlaybackStreamCallback=NULL;

    //DDC2StreamCallback is not required in this example
    Callbacks.DDC2StreamCallback=NULL;

    //DDC2StreamCallback is not required in this example
    Callbacks.DDC2StreamCallback=NULL;

    //DDC2PreprocessedStreamCallback is used to display signal level in this example
    Callbacks.DDC2PreprocessedStreamCallback=MyDDC2PreprocessedStreamCallback;

    //AudioStreamCallback is used to play audio on wave out device
    Callbacks.AudioStreamCallback=MyAudioStreamCallback;
    
    //AudioPlaybackStreamCallback is not used in this example
    Callbacks.AudioPlaybackStreamCallback=NULL;    

    //register callback functiobs
    SetCallbacks(hDevice,&Callbacks,0);

    //tune to an AM station frequency
    SetFrequency(hDevice,DDC2Id,702000);
    //it is possible to tune DDC1, DDC2 and demodulator directly using
    //SetDDCFrequency and SetDemodulatorFrequency
    //SetFrequency(hDevice,DDC2Id,702000) can be replaced by the following:
    //SetFrontEndFrequency(hDevice,0);
    //SetDDCFrequency(hDevice,DDC1Id,702000);
    //SetDDCFrequency(hDevice,DDC2Id,0);
    //SetDemodulatorFrequency(hDevice,DDC2Id,0);
    //final absolute frequency of the demodulator is 702000+0+0=702000

    //get absolute frequency where the demodulator is tuned
    GetFrequency(hDevice,DDC2Id,&Frequency);

    //get front-end frequency
    GetFrontEndFrequency(hDevice,&FrontEndFrequency);
    
    //get relative DDC1 frequency
    GetDDCFrequency(hDevice,DDC1Id,&DDC1Frequency);

    //get relative DDC2 frequency
    GetDDCFrequency(hDevice,DDC2Id,&DDC2Frequency);

    //get relative demodulator frequency
    GetDemodulatorFrequency(hDevice,DDC2Id,&DemodulatorFrequency);

    printf("Listening to (demodulator absolute frequency): %" PRIu64 " Hz\n",Frequency);
    printf("Front-end frequency: %" PRIu64 " Hz (range %d)\n",FrontEndFrequency,FrontEndFrequency==0?1:2);
    printf("DDC1 frequency (relative): %d Hz\n",DDC1Frequency);
    printf("DDC2 frequency (relative): %d Hz\n",DDC2Frequency);
    printf("Demodulator frequency (relative): %d Hz\n",DemodulatorFrequency);


    //set AGC params, attack=0.02s, decay=0.5s, ref. level=-15dB
    SetAGCParams(hDevice,DDC2Id,0.020,0.500,-15.0);

    //enable AGC
    SetAGC(hDevice,DDC2Id,1);

    //set AM mode
    SetDemodulatorMode(hDevice,DDC2Id,G69DDC_MODE_AM);

    //set demodulator filter bandwidth to 8kHz
    SetDemodulatorFilterBandwidth(hDevice,DDC2Id,8000);

    //calculate number of samples per DDC1 buffer - 100 ms
    GetDDC(hDevice,0,&DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start DDC1 streaming which has to be running before DDC2 is started
    StartDDC(hDevice,DDC1Id,SamplesPerBuffer);

    printf("\nPress ENTER to exit.\n\n");    

    //calculate number of samples per DDC2 buffer - 100 ms
    GetDDC(hDevice,DDC2Id,&DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
               
    //start DDC2 streaming, it has to be running before StartAudio is called
    StartDDC(hDevice,DDC2Id,SamplesPerBuffer);


    //calculate number of samples per audio buffer - 100 ms    
    //default audio sampling rate is 48000 Hz
    SamplesPerBuffer=100*AUDIO_SAMPLE_RATE/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start audio streaming
    StartAudio(hDevice,DDC2Id,SamplesPerBuffer);
    
    getchar();

    //stop audio streaming
    StopAudio(hDevice,DDC2Id);

    //stop DDC2 streaming
    StopDDC(hDevice,DDC2Id);

    //stop DDC1 streaming
    StopDDC(hDevice,DDC1Id);
    
    //unregister all callback functions
    SetCallbacks(hDevice,NULL,0);

    SetPower(hDevice,0);

    CloseDevice(hDevice);

    if(PaSimple)
    {
        pa_simple_free(PaSimple);
    }
    
    dlclose(API);

    printf("\n");
    
    return 0;
}
